#include <assert.h>
#include <string.h>

#include "libdg036z.h"


#define Mask(x) (1 << (7-(x & 0x07)))


BITARRAY::BITARRAY (int Width, int Height) :
  SizeX (Width),
  SizeY (Height)
{
  Cols = (int)(Width / 8 + 0.5);
  Data = new byte [Cols * SizeY];  

  memset (Data, 0x00, Cols * SizeY);
}


BITARRAY::~BITARRAY ()
{
  if (Data != NULL)
    delete Data;
}


void BITARRAY::Clear (bool Bit /* = 0 */)
{
  int  count;
  byte Value;

  Value = (Bit)? 0xff:0x00;

  for (count = 0; count < Cols * SizeY; count++)
    Data[count] = Value;
}


void BITARRAY::SetBit (int x, int y, bool Bit, OP Operation /* = SET */)
{
  int Offset;

  Offset = GetIndex(x,y);
  switch (Operation)
  {
    case CLEAR :
      Data[Offset] &= ~(Mask(x));
      break;

    case SET :
      if (Bit)
	Data[Offset] |= Mask (x);
      else
        Data[Offset] &= ~(Mask(x));
      break;

    case XOR :
      Data[Offset] ^= Mask (x);
      break;
  }
}  
 

bool BITARRAY::GetBit (int x, int y)
{
  return (Data[GetIndex(x,y)] & Mask(x));
}


int BITARRAY::GetIndex (int x, int y, int Base /* = 8 */)
{
  if (Base == 8)
    return (y * Cols + (x >> 3));
  else
    return (y * ((int)(SizeX/Base+0.5)) + (x % Base));
}


byte BITARRAY::GetByte (int x, int y, int Base /* = 8 */)
{
  int  count, Offs;
  byte Result = 0;
  
  assert (Base > 0 && Base <= 8);

  if (Base == 8)
    Result = Data[GetIndex(x,y,Base)];
  else
  {  
    Offs = (int)(x/Base) * Base;
    for (count = 0; count < Base; count++)
      if (GetBit(Offs+count, y))
	Result |= (1 << (5-count));
  }

  return Result;
}


