#include <sys/time.h>
#include <unistd.h>

#include <Xm/DrawingA.h>
#include <Xm/Label.h>

#include <libdlg.h>
#include <libprg.h>
#include <libhelp.h>

#include <libdg036z.h>

#include <icons/bomb.xpm>
#include <icons/image.xpm>

#include <icons/locomotive.xbm>
#include <icons/tux.xbm>


#define LPT_BASE 0x378

#define PIXEL_SIZE 4

#ifndef RAD
#define RAD(x) ((x) * 3.1415926535 / 180.0)
#endif


typedef struct
{
  int Number;
  int PosX[5];
  int PosY[5];
  int Height[3];
} HANOI;

typedef struct
{
  DG036Z  *LCD;
  Widget   widget;
  Window   Win;
  Pixmap   Pic;
} PANEL;

static HANOI Towers = {0, {0, 0, 0, 0, 0}, {0, 0, 0, 0, 0}, {0, 0, 0}};
static PANEL Panel = { NULL, NULL, None, None };


void SetPixel      (int x, int y);
void UpdateDisplay ();


static void ExitPrg ()
{
  if (Panel.LCD)
    delete Panel.LCD;

  PRGExitProgram ();
}


static void Clear ()
{
  if (Panel.LCD)
  {
    Panel.LCD->Clear ();
    UpdateDisplay ();
  }
  else
    PRGPrintf ("LCD not initialized");
}


#define AMPLITUDE 10

static void Draw ()
{
  int count;

  if (Panel.LCD == NULL) 
    PRGPrintf ("LCD not initialized");
  else
  {
    // graphic demo
    Panel.LCD->DrawLine (0, 24, 119, 24);
    Panel.LCD->DrawLine (60, 0, 60, 47);
    for (count = 5; count < 120; count+=5)
      Panel.LCD->DrawPoint (count, 25);
    for (count = 4; count < 48; count+=4)
      Panel.LCD->DrawPoint (59, count);

    for (count = 1; count < 120; count++)
    {
      Panel.LCD->DrawPoint (count, 24 - sin(6*count*3.1415/180.0) * 15.0);
      Panel.LCD->DrawPoint (count, 24 - cos(6*count*3.1415/180.0) * 15.0);
    }

    Panel.LCD->DrawLine (  0, 24,           14,  24+AMPLITUDE);
    Panel.LCD->DrawLine ( 14, 24+AMPLITUDE, 44,  24-AMPLITUDE);
    Panel.LCD->DrawLine ( 44, 24-AMPLITUDE, 74,  24+AMPLITUDE);
    Panel.LCD->DrawLine ( 74, 24+AMPLITUDE, 104, 24-AMPLITUDE);
    Panel.LCD->DrawLine (104, 24-AMPLITUDE, 119, 24);

    Panel.LCD->Refresh ();
    UpdateDisplay ();
  }
}


static void SetMode (int Mode)
{
  switch (Mode)
  {
     case 2 : Panel.LCD->SetMode (DG036Z::TEXT);
              break;
     case 3 : Panel.LCD->SetMode (DG036Z::GRAPHIC);
              break;
     case 4 : Panel.LCD->SetMode (DG036Z::TEXT | DG036Z::GRAPHIC);
              break;
     case 1 : 
    default : Panel.LCD->SetMode (DG036Z::OFF);
  }
  UpdateDisplay ();
}


#define STEP 1

static void FadeOut (int Mode)
{
  int count;
  int countX, countY;

  switch (Mode)
  {
    case 0 :
      Panel.LCD->Clear ();
      break;

    case 1 :
      for (countY = 0; countY < 48; countY+=2) 
      {
	Panel.LCD->DrawLine (0, countY, 119, countY, T6963C::CLEAR);
	UpdateDisplay ();
      }
      for (countY = 47; countY > 0; countY-=2)
      {
	Panel.LCD->DrawLine (0, countY, 119, countY, T6963C::CLEAR);
	UpdateDisplay ();
      }
      break;

    case 2 :
      // Clear Display
      for (count = 0; count <= sqrt(60*60 + 24*24); count++)
      {
	Panel.LCD->FillArc (60, 24, count, count, 0, 360, 
                            T6963C::SET, T6963C::CLEAR);
	UpdateDisplay ();
      }
      break;

    case 3 :
      for (countY = 0; countY < 24; countY++)
      {
	Panel.LCD->DrawLine (0,    countY, 119,    countY, T6963C::CLEAR);
	Panel.LCD->DrawLine (0, 47-countY, 119, 47-countY, T6963C::CLEAR);
	UpdateDisplay ();
      }
      break;

    case 4 :
      {
        int left = 0, right = 119;
	int top = 0, bottom = 47;

	while (left < right && top < bottom)
	{
	  for (countY = 0; countY < STEP; countY++)
	  {
	    Panel.LCD->DrawLine (left, top, right, top);
	    top++;
	  }
	  UpdateDisplay ();
	
	  for (countX = 0; countX < STEP; countX++)
	  {
	    Panel.LCD->DrawLine (right, top, right, bottom);
	    right--;
	  }
	  UpdateDisplay ();
	  
	  for (countY = 0; countY < STEP; countY++)
	  {
	    Panel.LCD->DrawLine (right, bottom, left, bottom);
	    bottom--;
	  }
	  UpdateDisplay ();

	  for (countX = 0; countX < STEP; countX++)
	  {
	    Panel.LCD->DrawLine (left, bottom, left, top);
	    left++;
	  }
	  UpdateDisplay ();
	}
      }
      break;

    case 5 :
      for (countY = 0; countY < 48; countY++)
      {
	for (countX = 0; countX < 120; countX++)
	  if ((countX+countY) & 1)
	    Panel.LCD->DrawPoint (countX, countY, T6963C::SET);
	UpdateDisplay ();
      }
      for (countX = 0; countX < 120; countX++)
      {
	for (countY = 0; countY < 48; countY++)
	  if (! ((countX+countY) & 1))
	    Panel.LCD->DrawPoint (countX, countY, T6963C::SET);
	UpdateDisplay ();
      }
      Panel.LCD->Clear ();
      break;
  }
}


static void ShowText ()
{
  static char    Text[120] = "";
         DIALOG *Dlg;

  if (Panel.LCD)
  {
    Dlg = DLGNewDialog (0);
    DLGInsertText (Dlg, 0, "text", 120, Text, 20, 6);
    DLGOpenDialog (Dlg, "dialog", DlgNORMAL, NULL);
    if (DLGGetExitButton(Dlg) == DlgOK)
    {
      strncpy (Text, DLGGetText(Dlg, 0), 120);
      Panel.LCD->ShowText (Text);
      UpdateDisplay ();
    }
    DLGFreeDialog (Dlg);
  }
  else
    PRGPrintf ("LCD not initialized");
}


static void FloatText (char *Text)
{
  int         count, Len;
  static char View[20];

  Len = strlen (Text);
  Text[Len++] = ' ';
  Text[Len] = '\0';
  for (count = 0; count < Len; count++)
  {
    strncpy (View, Text+count, MIN(20, Len-count));
    Panel.LCD->ShowText (View);
    UpdateDisplay ();

    TOOLWait (250000);
  }
}

static void FloatTextDlg ()
{
  static char    Text[121] = "";
         DIALOG *Dlg;

  if (Panel.LCD)
  {
    Dlg = DLGNewDialog (0);
    DLGInsertText (Dlg, 0, "text", 120, Text, 20, 1);
    DLGOpenDialog (Dlg, "dialog", DlgNORMAL, NULL);
    if (DLGGetExitButton(Dlg) == DlgOK)
    {
      strncpy (Text, DLGGetText(Dlg, 0), 120);
      FloatText (Text);
    }
    DLGFreeDialog (Dlg);
  }
}


static void ShowGraphic (int No)
{
  if (Panel.LCD == NULL)
    return;

  switch (No)
  {
    case 1 :
      Panel.LCD->ShowImage (locomotive_bits, 
                            locomotive_width, locomotive_height); 
      UpdateDisplay ();
      break;
    case 2 :
      Panel.LCD->ShowImage (tux_bits, tux_width, tux_height);
      UpdateDisplay ();
      break;
    default :
      {
	         int  SizeX, SizeY;
	unsigned char *Data;
	  static char FileName[PATH_MAX+1] = 
	                "/users/stefan/project/src/include/icons/p500.xbm";
	         char Mask[PATH_MAX+1] = "*.xbm";
  
	if (DLGOpenFileSel("loadfile", FileName, PATH_MAX, Mask) != NULL)
        {
          Data = TOOLLoadXBMData(FileName, &SizeX, &SizeY, NULL, NULL);
          Panel.LCD->ShowImage (Data, SizeX, SizeY);
        }	 
      }
  }
}


void SetCursor (int Mode)
{
  if (Panel.LCD)
    Panel.LCD->SetCursorPattern (Mode-1);
}


static byte Segment[2][15] = 
{
  {0x07, 0xFF, 0x00,
   0x0F, 0xFF, 0x80,
   0x1F, 0xFF, 0xC0,
   0x0F, 0xFF, 0x80,
   0x07, 0xFF, 0x00},
  {0x20,
   0x70,
   0xF8,
   0xF8, 
   0xF8,
   0xF8,
   0xF8,
   0xF8,
   0xF8,
   0xF8,
   0xF8,
   0xF8,
   0xF8,
   0x70,
   0x20}
};

static int SegmentGeo[7][3] =   // Type, x, y
{
  {0,  0,  2},   // a
  {1, 16,  5},   // b
  {1, 16, 21},   // c
  {0,  0, 34},   // d
  {1,  0, 21},   // e
  {1,  0,  5},   // f
  {0,  0, 18}    // g
};

static int SegmentPos[10][7] =
{//a  b  c  d  e  f  g
  {1, 1, 1, 1, 1, 1, 0},  // 0
  {0, 1, 1, 0, 0, 0, 0},  // 1
  {1, 1, 0, 1, 1, 0, 1},  // 2
  {1, 1, 1, 1, 0, 0, 1},  // 3
  {0, 1, 1, 0, 0, 1, 1},  // 4
  {1, 0, 1, 1, 0, 1, 1},  // 5
  {1, 0, 1, 1, 1, 1, 1},  // 6
  {1, 1, 1, 0, 0, 0, 0},  // 7
  {1, 1, 1, 1, 1, 1, 1},  // 8
  {1, 1, 1, 1, 0, 1, 1}   // 9
};


static void ShowDigit (byte Value, int Pos)
{
  int count, Index;
  int Bit;

  for (count = 0; count < 7; count++)
    if (SegmentPos[Value][count])
      switch (SegmentGeo[count][0])
      {
        case 0 :
	  for (Index = 0; Index < 15; Index++)
	    for (Bit = 0; Bit < 8; Bit++)
	      if (Segment[0][Index] & (1 << (7-Bit)))
		Panel.LCD->DrawPoint (Pos+SegmentGeo[count][1]+8*(Index%3)+Bit,
				      SegmentGeo[count][2]+Index/3);
	  break;

        case 1 :
	  for (Index = 0; Index < 15; Index++)
	    for (Bit = 0; Bit < 8; Bit++)
	      if (Segment[1][Index] & (1 << (7-Bit)))
		Panel.LCD->DrawPoint (Pos+SegmentGeo[count][1]+Bit,
				      SegmentGeo[count][2]+Index);
	  break;	  
      }
}


void Digit (int Value, dual counter)
{
  int count, Digit;

  Panel.LCD->SetRefreshMode (0);

  do
  {
    Panel.LCD->Clear ();

    ShowDigit (Value % 10, 96);
    Digit = Value / 10;

    for (count = 1; count < 5; count++)
    {
      if (Digit > 0)
	ShowDigit (Digit % 10, 96-count*24);
      Digit /= 10;
    }
    Panel.LCD->Refresh ();
    UpdateDisplay ();

    if (counter)
      Value--;
    else
      Value = 0;
  } while (Value > 0);

  Panel.LCD->Transfer ();
}


void DigitDlg ()
{
	  int     Button;
  static  int     Value = 0;
  DIALOG *Dlg;

  Dlg = DLGNewDialog (0);
  DLGInsertInt (Dlg, 0, "numer", Value, 5);
  DLGMakeButton (Dlg, OK_BUTTON);
  DLGMakeButton (Dlg, USER_ID, "count", NULL, -5);
  DLGMakeButton (Dlg, CANCEL_BUTTON);
  DLGOpenDialog (Dlg, "setNumber", DlgNORMAL, NULL);
  if ((Button = DLGGetExitButton(Dlg)) != DlgCANCEL)
  {
    Value = DLGGetInt (Dlg, 0);

    Digit (Value, Button != DlgOK);
  }

  DLGFreeDialog (Dlg);
}


void SetPixel (int x, int y, int Value)
{
  if (x < 0 || x >= 120 || y < 0 || y >= 48)
    return;

  if (Value)
    XSetForeground (XDisplay, copyGC, TOOLGetColor("DarkBlue"));
  else
    XSetForeground (XDisplay, copyGC, TOOLGetColor("Green"));
  XFillRectangle (XDisplay, Panel.Pic, copyGC, x*PIXEL_SIZE, y*PIXEL_SIZE, 
		                               PIXEL_SIZE-1, PIXEL_SIZE-1);
}


void UpdateDisplay ()
{
  int       countX, countY;
  BITARRAY *Img;

  Img = Panel.LCD->GetImage ();

  for (countY = 0; countY < 48; countY++)
    for (countX = 0; countX < 120; countX++)
      SetPixel (countX, countY, Img->GetBit(countX, countY));

  XCopyArea (XDisplay, Panel.Pic, XtWindow(Panel.widget), copyGC, 0, 0, 
	     120 * PIXEL_SIZE, 48 * PIXEL_SIZE, 0, 0);
}


static void PanelHandler (Widget widget, void *Data, XEvent *Event)
{
  static int Btn = 0;

  switch (Event->type)
  {
    case Expose :
      {
	XExposeEvent *expose = (XExposeEvent *)Event;

	XCopyArea (XDisplay, Panel.Pic, expose->window, copyGC,
		   expose->x, expose->y,  expose->width, expose->height,
		   expose->x, expose->y);
      }
      break;

    case MotionNotify :
      {
	XMotionEvent *Motion = (XMotionEvent *)Event;
	int           PosX, PosY;

	if (Btn != 0)
	{
	  PosX = Motion->x / PIXEL_SIZE;
	  PosY = Motion->y / PIXEL_SIZE;
	  Panel.LCD->DrawPoint (PosX, PosY, 
			        (Btn==1)?T6963C::SET : T6963C::CLEAR);
#if 1
	  UpdateDisplay ();
	  Panel.LCD->Transfer ();
#else
	  SetPixel (PosX, PosY, 2-Btn);
	  XCopyArea (XDisplay, Panel.Pic, XtWindow(Panel.widget), copyGC, 
		     PosX*PIXEL_SIZE, PosY*PIXEL_SIZE, 
		     PIXEL_SIZE-1, PIXEL_SIZE-1, 
		     PosX*PIXEL_SIZE, PosY*PIXEL_SIZE);
#endif
	}
      }
      break;

    case ButtonPress :  
      {
	XButtonEvent *Button = (XButtonEvent *)Event;

	Btn = Button->button;
      }
      break;

    case ButtonRelease :
      Btn = 0;
      break;

    case VisibilityNotify :
      if (Panel.Pic == None)
      {
	Panel.Pic = XCreatePixmap (XDisplay, XtWindow(widget), 
				   120*PIXEL_SIZE, 48*PIXEL_SIZE, 
				   TOOLGetDepth());
	XSetForeground (XDisplay, copyGC, TOOLGetColor("Green"));
	XFillRectangle (XDisplay, Panel.Pic, copyGC, 
			0, 0, 120*PIXEL_SIZE, 48*PIXEL_SIZE);
      }
      break;
  }
}


void Histo ()
{
  int count, Bar;
  int OffsX;
  int Val[12];

  for (count = 0; count < 12; count++)
    Val[count] = 22;

  Panel.LCD->SetRefreshMode (0);

  for (count = 0; count < 100; count++)
  {
    Panel.LCD->Clear ();

    for (Bar = 0; Bar < 12; Bar++)
    {
      if (rand() > 0.5 * RAND_MAX)
	Val[Bar] = MIN (48, Val[Bar]+1);
      else
	Val[Bar] = MAX (2, Val[Bar]-1); 
    
      if (Val[Bar] < 48)
      {
	OffsX = 1 + Bar * 10;
	Panel.LCD->FillRectangle (OffsX, Val[Bar], OffsX+5, 47);
	Panel.LCD->DrawPoint (OffsX+1, Val[Bar]-1);
	Panel.LCD->DrawPoint (OffsX+6, Val[Bar]-1);
	Panel.LCD->DrawLine  (OffsX+2, Val[Bar]-2, OffsX+7, Val[Bar]-2);
	Panel.LCD->DrawLine  (OffsX+7, Val[Bar]-2, OffsX+7, 47);
      }
    }
    Panel.LCD->Refresh ();
    UpdateDisplay ();
    TOOLWait (50000);               // slow down motion
  }
  Panel.LCD->SetRefreshMode (1);
}


void Move ()
{
  int count;

  Panel.LCD->SetRefreshMode (1);
  for (count = 0; count < 120; count++)
  {
    Panel.LCD->Move (1, 0);
    UpdateDisplay ();
  }
}


void Clock ()
{
         int     Hour, Min, Sec; 
	 double  Angle[3] = {0, 0, 0};
  struct timeval Start, Time, Rest;

  SetMode (4);
  Panel.LCD->SetRefreshMode (0);

  gettimeofday (&Start, NULL);

  Panel.LCD->ShowText ("Time:");

  do
  {
    gettimeofday (&Time, NULL);
    Hour = (Time.tv_sec / 3600) % 12;
    Min  = (Time.tv_sec % 3600) / 60;
    Sec  = (Time.tv_sec % 60);

    Panel.LCD->FillArc (60, 24, 22, 22, 0, 360, T6963C::SET, T6963C::CLEAR);
    
    Angle[0] = RAD (90.0 - 30.0*Hour);
    Panel.LCD->DrawLine (60, 24, 60+cos(Angle[0])*12, 24-sin(Angle[0])*12);

    Angle[1] = RAD (90.0 - 6.0*Min);
    Panel.LCD->DrawLine (60, 24, 60+cos(Angle[1])*18, 24-sin(Angle[1])*18);

    Angle[2] = RAD (90.0 - 6.0*Sec);
    Panel.LCD->DrawLine (60, 24, 60+cos(Angle[2])*20, 24-sin(Angle[2])*20);

    Panel.LCD->Refresh ();
    UpdateDisplay ();

    // Wait till next second
    gettimeofday (&Rest, NULL);
    TOOLWait (1000000 - (Rest.tv_usec % 1000000));
  } while ((Time.tv_sec - Start.tv_sec) < 30);
}



static void ShowHanoi ()
{
  int count, Size;

  Panel.LCD->Clear ();

  Panel.LCD->FillRectangle ( 0, 45, 119, 47, T6963C::SET);
  Panel.LCD->FillRectangle (29, 15,  31, 44, T6963C::SET);
  Panel.LCD->FillRectangle (59, 15,  61, 44, T6963C::SET);
  Panel.LCD->FillRectangle (89, 15,  91, 44, T6963C::SET);

  for (count = 0; count < Towers.Number; count++)
  {
    Size = 2 + (count+1) * 2;
    Panel.LCD->FillRectangle (Towers.PosX[count]-Size, Towers.PosY[count],
			      Towers.PosX[count]+Size, Towers.PosY[count]+4,
			      T6963C::SWAP);
  }

  Panel.LCD->Refresh ();
  UpdateDisplay ();
}

static void HanoiMove (int From, int To, int Size)
{
  int count;
  int Pos1, Pos2;

  Pos1 = (From+1) * 30;
  Pos2 = (To+1)   * 30;

  // Pick up slice (depart)
  for (count = 40-Towers.Height[From]*5; count >= 6; count--)
  {
    Towers.PosY[Size-1] = count;
    ShowHanoi ();
  }

  // Move slice to target stack
  count = 0;
  do
  {
    count += (Pos2 > Pos1) ? 1:(-1);  
    Towers.PosX[Size-1] = Pos1+count;
    ShowHanoi ();
  } while (abs(count) < abs(Pos2-Pos1));

  // place slice 
  for (count = 7; count <= 40-Towers.Height[To]*5; count++)
  {
    Towers.PosY[Size-1] = count;
    ShowHanoi ();
  }
}

static void Hanoi (int n, int From, int Use, int To)
{
  if (n > 0)
  {
    // Move n-1 slices from position 'From' to position 'Use'
    Hanoi  (n-1, From, To, Use);

    // Move the left n-th slice to the goal position 'To' 
    PRGPrintf ("#%d: %d -> %d | Size: [%d] [%d] [%d]", n, From+1, To+1, 
	       Towers.Height[0], Towers.Height[1], Towers.Height[2]);

    // Move the slice
    HanoiMove (From, To, n);

    // update stack-height
    (Towers.Height[From])--;
    (Towers.Height[To])++;

    // Put the saved n-1 slices from position 'Use' to goal-position 'To'
    Hanoi  (n-1, Use, From, To);
  }
}

static void HanoiDlg (void *Data)
{
  int     No, count;
  DIALOG *Dlg;

  if ((No = (int)Data) <= 0)
  {
    Dlg = DLGNewDialog (0);
    DLGInsertUpDn (Dlg, 0, "numer", 2, 5, 3, 1);
    DLGOpenDialog (Dlg, "setNumber", DlgNORMAL, NULL);
    if (DLGGetExitButton(Dlg) == DlgOK)
      Towers.Number = (int)DLGGetUpDn (Dlg, 0);
    DLGFreeDialog (Dlg);
  }
  else
    Towers.Number = No;

  Panel.LCD->Clear ();
  Panel.LCD->SetRefreshMode (0);

  // Initialize data of slices and stacks...
  for (count = 0; count < Towers.Number; count++)
  {
    Towers.PosX[count] = 30;
    Towers.PosY[count] = 40 - (Towers.Number-count-1) * 5;
  }
  Towers.Height[0] = Towers.Number;
  Towers.Height[1] = 0;
  Towers.Height[2] = 0;

  Hanoi (Towers.Number, 0, 1, 2);     // Call recursive algorithm

  Panel.LCD->SetRefreshMode (1);
}


#define FUNCTION0(x,y) (1.0 / ((x)*(x) + (y)*(y) + 0.5))
#define FUNCTION1(x,y) (1.5 * sin(2*(x)) * cos(2*(y)))

static void Plot3D (int No)
{
  int    count;
  int    pz;
  int    ui, vi;
  double x, y, z;
  double u, v;
  double si, co;
  double len, u0, v0;
  double lx, ly, rx, ry;
  double sx, sy;
  double Max[120];

  SetMode (4);
  
  Panel.LCD->Clear ();
  Panel.LCD->SetRefreshMode (0);

  Panel.LCD->ShowText ("Math:");

  lx = -2.0;
  rx =  2.0;
  ly = -2.0;
  ry =  2.0;

  si  = sin (RAD(30.0));
  co  = cos (RAD(30.0));
  len = rx - lx;
  pz  = int (120.0 / len);
  u0  = -lx * pz;
  v0  = 39 + ly * si * pz;
  sx  = len / 120.0 * 2;
  sy  = sx * si * 2;

  for (count = 0; count < 120; count++)
    Max[count] = 39;

  y = ly;
  while (y < ry)
  {
    x = lx;
    while (x < rx)
    {
      switch (No)
      {
         case 0 : z = FUNCTION0 (x, y);
	          break;
         case 1 : z = FUNCTION1 (x, y);
	          break;
        default : z = FUNCTION0 (1.2*x, 1.2*y) * cos(5.0 * (x*x + y*y));
      }

      u = x + y * co;
      v = z + y * si;
      ui = int (u0 + u * pz + 0.5);
      vi = int (0.5 * (v0 - v * pz)) + 25;

      if (ui >= 0 && ui < 120 && vi >= 0 && vi < 39)
      {
	if (vi < Max[ui])
	  Max[ui] = vi;
	if (vi <= Max[ui])
	  Panel.LCD->DrawPoint (ui, vi + 5);
      }

      x += sx;
    }
    UpdateDisplay ();
    Panel.LCD->Refresh ();

    y += sy;
  }
}

static void Plot3DDlg ()
{
  int     No = -1;
  DIALOG *Dlg;

  Dlg = DLGNewDialog (0);
  DLGInsertRBox (Dlg, 0, "function", 1, 3, "1/sqrt", "sin*cos", "dist",
		                           0);
  DLGOpenDialog (Dlg, "selectFunc", DlgNORMAL, NULL);
  if (DLGGetExitButton(Dlg) == DlgOK)
    No = DLGGetRBox (Dlg, 0);      
  DLGFreeDialog (Dlg);

  if (No >= 0)
    Plot3D (No);
}


static void Demo ()
{
  int  count, Index;
  char Text[20] = "Test finished";

  SetMode (4);

  Panel.LCD->Clear ();

  Panel.LCD->DrawRectangle (1, 1, 118, 46);
  Panel.LCD->DrawRectangle (3, 3, 116, 44);
  Panel.LCD->FillRectangle (11, 8, 109, 26);
  Panel.LCD->FillRectangle ( 9, 6, 107, 23, T6963C::SET, T6963C::CLEAR);
  Panel.LCD->ShowText ("\n"
		       "   T O S H I B A\n"
		       "  Display  T6963C\n"
		       "\n"
		       "    loading ...");
  Panel.LCD->Transfer ();
  Panel.LCD->Clear (T6963C::TEXT);
  UpdateDisplay ();
  // Draw progress-display
  Panel.LCD->DrawRectangle (9, 31, 110, 39);
  for (count = 10; count < 110; count++)
  {
    Panel.LCD->DrawLine (count, 32, count, 38, T6963C::SWAP);
    UpdateDisplay ();
  }
  FadeOut (4);

  // Show Image
  ShowGraphic (1);

  count = 2000000;
  while (count > 1000)
  {
    TOOLWait (count);
    Panel.LCD->FillRectangle (0, 0, 119, 47, T6963C::SET, T6963C::SWAP);
    UpdateDisplay ();
    count = (int)(0.5 * count);
  }

  FadeOut (1);

  // draw frames
  for (count = 0; count <= 22; count+=2)
    Panel.LCD->DrawRectangle (count, count, 119-count, 47-count);  

  // remove frames
  for (Index = 22; Index >= 0; Index-=2)
    for (count = 0; count < 120-2*Index+1; count+=8) 
     {
      Panel.LCD->Move (1, 0, Index, Index, 119-Index, 47-Index);
      UpdateDisplay ();
    }
  for (count = 0; count < 120; count+=2)
  {
    Panel.LCD->Move (2, 0, 0, 0, 119, 47);
    UpdateDisplay ();
  }    

  // Plot functions
  Draw ();
  TOOLWait (4000000);
  FadeOut (2);

  // Show Histogramms
  Histo ();

  for (count = 22; count >= 1; count--)
  {
    Panel.LCD->FillRectangle (count, count, 119-count, 47-count, 
			      T6963C::SET, T6963C::CLEAR);
    UpdateDisplay ();
  }

  // plot math function
  Plot3D (0);
  Move ();

  // Display digits
  Digit (100, TRUE);
  Panel.LCD->Clear ();

  // Draw simple watch
  Clock ();
  Panel.LCD->Clear ();

  // Towers of hanoi
  HanoiDlg ((void*)3);
  FadeOut (5);

  // Move text
  FloatText (Text);

  Panel.LCD->ShowText ("\n"
		       "  Test finished !\n"
		       "\n"
		       " Have a lot of fun");
  UpdateDisplay ();

#if 0
  for (count = 17; count >= 2; count--)
  {
    strncpy (Text, "  Test finished !!!  ", 18-count);
    Panel.LCD->Print (count, 1, Text);
    UpdateDisplay ();
    TOOLWait (250000);
  }
#endif

  FadeOut (3);
}


static void Control ()
{
  Panel.LCD->SetMode (DG036Z::TEXT | DG036Z::GRAPHIC);
  Panel.LCD->Clear   ();

  Panel.LCD->ShowText ("Temp.: 20 C  Speed:\n"
		       "             1200Mhz"
                       "RPM: 2000 ID\n"
		       "          06\n"
                       "Voltage:      4,9 V "
		       "             12,1 V");

  Panel.LCD->DrawLine (77,  0, 77, 47);
  Panel.LCD->DrawLine ( 0, 10, 76, 10);
  Panel.LCD->DrawLine (57, 11, 57, 47);

  Panel.LCD->Refresh ();
  UpdateDisplay ();
}


static void HelpMe ()
{
}


void InitDisplay (Widget Work)
{
  Panel.LCD = new DG036Z (LPT_BASE);
  Panel.LCD->SetMode (DG036Z::TEXT | DG036Z::GRAPHIC);

  Panel.widget = XtVaCreateManagedWidget ("panel", 
					  xmDrawingAreaWidgetClass, Work,
					  XmNwidth, 120*PIXEL_SIZE,
					  XmNheight, 48*PIXEL_SIZE,
					  NULL);
  
  XtAddEventHandler (Panel.widget,
		     ExposureMask | EnterWindowMask | LeaveWindowMask | 
		     PointerMotionMask | ButtonPressMask | ButtonReleaseMask |
 		     StructureNotifyMask | VisibilityChangeMask, 
		     FALSE, 
		     (XtEventHandler)PanelHandler, NULL);
  Panel.Win = XtWindow (Panel.widget);
}


static MENU_ENTRY SystemMenu[] = 
{
  ENTRY ("exit",  "",        "",   ' ', TRUE, ExitPrg, NULL)
};

static MENU_ENTRY TextMenu[] = 
{
  ENTRY    ("text",     "",        "",   ' ', TRUE, ShowText,       NULL),  
  ENTRY    ("float",    "",        "",   ' ', TRUE, FloatTextDlg,   NULL)
};

static MENU_ENTRY GraphicMenu[] = 
{
  ENTRY ("p500", "",         "",   ' ', TRUE, ShowGraphic, 1),
  ENTRY ("tux",  "",         "",   ' ', TRUE, ShowGraphic, 2),
  MENU_LINE (PrgETCHED_IN),
  ENTRY ("load", "",         "",   ' ', TRUE, ShowGraphic, 3)
};

static MENU_ENTRY CursorMenu[] = 
{
  ENTRY ("line1", "",        "",   ' ', TRUE, SetCursor, 1),
  ENTRY ("line2", "",        "",   ' ', TRUE, SetCursor, 2),
  ENTRY ("line3", "",        "",   ' ', TRUE, SetCursor, 3),
  ENTRY ("line4", "",        "",   ' ', TRUE, SetCursor, 4),
  ENTRY ("line5", "",        "",   ' ', TRUE, SetCursor, 5),
  ENTRY ("line6", "",        "",   ' ', TRUE, SetCursor, 6),
  ENTRY ("line7", "",        "",   ' ', TRUE, SetCursor, 7),
  ENTRY ("line8", "",        "",   ' ', TRUE, SetCursor, 8)
};

static MENU_ENTRY ModeMenu[] =
{
  ENTRY ("off",         "",        "",   ' ', TRUE, SetMode, 1),
  ENTRY ("text",        "",        "",   ' ', TRUE, SetMode, 2),
  ENTRY ("graphic",     "",        "",   ' ', TRUE, SetMode, 3),
  ENTRY ("textGraphic", "",        "",   ' ', TRUE, SetMode, 4)
};

static MENU_ENTRY DisplayMenu[] = 
{
  ENTRY     ("clear",    "",        "",   ' ', TRUE, Clear,       NULL),
  SUB_MENU  ("text",     "",        "",   ' ', TRUE, TextMenu),
  SUB_MENU  ("graphic",  "",        "",   ' ', TRUE, GraphicMenu),
  SUB_MENU  ("cursor",   "",        "",   ' ', TRUE, CursorMenu),
  ENTRY     ("draw",     "",        "",   ' ', TRUE, Draw,        NULL),
  ENTRY     ("histo",    "",        "",   ' ', TRUE, Histo,       NULL),
  ENTRY     ("digit",    "",        "",   ' ', TRUE, DigitDlg,    NULL),
  SUB_MENU  ("mode",     "",        "",   ' ', TRUE, ModeMenu),
  ENTRY     ("move",     "",        "",   ' ', TRUE, Move,        NULL),
  ENTRY     ("clock",    "",        "",   ' ', TRUE, Clock,       NULL),
  ENTRY     ("hanoi",    "",        "",   ' ', TRUE, HanoiDlg,    -1),
  ENTRY     ("plot3d",   "",        "",   ' ', TRUE, Plot3DDlg,   NULL),
  ENTRY     ("control",  "",        "",   ' ', TRUE, Control,     NULL),
  MENU_LINE (PrgETCHED_IN),
  ENTRY     ("demo",     "",        "",   ' ', TRUE, Demo,        NULL)
};


static MENU_ENTRY HelpMenu[] =
{
  ENTRY     ("help",  "<Key>F1", "F1", 'h', TRUE, HelpMe,    NULL)
};

static MENU_COLUMN MenuCols[] = 
{
  FUNC_MENU ("system",  's', TRUE, SystemMenu),
  FUNC_MENU ("display", 'd', TRUE, DisplayMenu),
  HELP_MENU ("help",    'h', TRUE, HelpMenu)
};

static MENU_BAR Menu = MAIN_MENU ("mainMenu", MenuCols);

       
int main (int argc, char **argv)
{
  Widget Work;

  PRGInitLib (argv[0], &argc, argv, NULL);

  PRGMakeToolBar (PrgTOP_TOOL);

  TOOL_ICON (bomb_xpm,  ExitPrg,     NULL, "Exit program");
  TOOL_SPACE (5);
  TOOL_ICON (image_xpm, ShowGraphic, 3,    "Load Image");
  // TextMode
  // Image/Text

  Work = PRGCreateProgram (argv[0], &Menu);
  InitDisplay (Work);

  PRGSetSize (0, 0);

  PRGStartProgram (NULL, NULL);
}





